 /*
 * Copyright (C) 2006-2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
*/


#ifndef INCL_WINDOWS_SYNC_SOURCE
#define INCL_WINDOWS_SYNC_SOURCE

#include "base/fscapi.h"
#include "spds/constants.h"
#include "spds/SyncItem.h"
#include "spds/SyncMap.h"
#include "spds/SyncStatus.h"
#include "spds/SyncSource.h"
#include"WindowsSyncSourceConfig.h"
#include "outlook/ClientApplication.h"

#include <string>
#include <list>


typedef std::list<std::wstring>         itemKeyList;
typedef itemKeyList::iterator           itemKeyIterator;

//
// Error codes for WindowsSyncSource:
// Codes  < 100 = soft errors -> SOURCE_ERROR not required
// Codes >= 100 = bad  errors -> after 10 bad errors 'state' = SOURCE_ERROR
//
#define ERR_CODE_FOLDER_PATH              1     // Item ignored (path not correct for current sync)
#define ERR_CODE_DELETE_NOT_FOUND         2     // Item not deleted cause not found
#define ERR_CODE_OLD_ITEMS_PATH         100
#define ERR_CODE_UPDATE_NOT_FOUND       101
#define ERR_CODE_FOLDER_OPEN            102
#define ERR_CODE_ITEM_CREATE            104
#define ERR_CODE_ITEM_FILL              105
#define ERR_CODE_ITEM_SAVE              106
#define ERR_CODE_DELETE                 107
#define ERR_CODE_ITEM_BAD_TYPE          108
#define ERR_CODE_ITEM_GET               109
#define ERR_CODE_NO_ROOT_FOLDER         110
#define ERR_CODE_OPEN_OUTLOOK           111

#define MAX_SOURCE_ERRORS                10     // Max number of bad errors allowed

#define TIMESTAMP_DELAY                   2     // Outlook introduces some delay while saving items


/*
 ******************************** WindowsSyncSource class ********************************
 *
 *
 *****************************************************************************************
 */
class WindowsSyncSource : public SyncSource {

private:

    // Pointer to the sole instance of ClientApplication (singleton)
    ClientApplication* outlook;


protected:

    // Configuration object for the source. It's a reference to WindowsSyncSourceConfig
    // object owned by OutlookConfig. It's automatically initialized in the constructor.
    WindowsSyncSourceConfig& winConfig;


    // Lists of item keys.
    itemKeyList  allItems;
    itemKeyList  newItems;
    itemKeyList  modItems;
    itemKeyList  delItems;
    itemKeyList  allItemsPaths;         // This is the list of all items path in Outlook

    // Iterators of lists.
    itemKeyIterator  iAll;
    itemKeyIterator  iNew;
    itemKeyIterator  iMod;
    itemKeyIterator  iDel;
    itemKeyIterator  iAllPaths;


    // Counter of number of errors occurred
    unsigned int numErrors;

    // Default folder path in Outlook
    std::wstring defaultFolderPath;


    // Internal utility methods:
    // -------------------------
    // Get the starting Outlook folder for this source.
    ClientFolder* getStartFolder();

    // Get all items inside 'folder' and push them (only keys) into 'listItems' list.
    void pushAllSubfolderItemsToList(ClientFolder* folder, itemKeyList& listItems, itemKeyList& listItemsPaths);
    void pushAllItemsToList         (ClientFolder* folder, itemKeyList& listItems, itemKeyList& listItemsPaths);

    // Fill internal itemKeyLists of NEW/MOD/DEL item keys from last successfull sync.
    int manageModificationsFromLastSync();

    // Add forced modified items (passed list) to the modItems list.
    int addForcedItemsToModList(itemKeyList& forcedItems);

    // Create/parse XML string with old item's keys.
    std::wstring createOldItems();
    void parseOldItems(std::wstring& data, itemKeyList& listItems, itemKeyList& listFolders);

    // Reset and close the data file (where current items ID are stored).
    int resetDataFile(const std::wstring& itemType);
    int closeDataFile(const std::wstring& itemType);

    // Returns true if path passed is allowed to sync items inside it.
    bool folderPathAllowed(const std::wstring& p);

    // Verifies if birthday/anniversary were created by Outlook (only for contacts save).
    void checkBirthdayAnniversary(ClientItem* cItem);
    int  deleteAppointment(ClientItem* cItem, const std::wstring& propertyName);

    // Common actions to do when an error occurs inside SyncSource.
    void manageSourceError(const int errorCode, const char* errorMsg);

    void extractFolder(const std::wstring dataString, const std::wstring dataType, std::wstring& path);


public:

    /**
     * Constructor: creates a WindowsSyncSource with the specified name and config
     *
     * @param name - the name of the SyncSource
     * @param wsc  - to init by reference the 'winConfig' member (for client-specific props)
     *               'config' member is initialized from wsc->getCommonConfig() to get the original
     *               common properties used by API.
     */
    WindowsSyncSource::WindowsSyncSource(const WCHAR* name, WindowsSyncSourceConfig* wsc);

    ~WindowsSyncSource();


    // Used to access configuration of SS
    const WindowsSyncSourceConfig& getConfig() const;
    WindowsSyncSourceConfig& getConfig();



    int beginSync();
    int endSync();

    void WindowsSyncSource::setItemStatus(const WCHAR* key, int status);

    void assign(WindowsSyncSource& s);

    ArrayElement* clone();



    /*
    * Return the first SyncItem of all. It is used in case of slow or refresh sync
    * and retrieve the entire data source content.
    */
    SyncItem* getFirstItem();

    /*
    * Return the next SyncItem of all. It is used in case of slow or refresh sync
    * and retrieve the entire data source content.
    */
    SyncItem* getNextItem();

    /*
    * Return the first SyncItem of new. It is used in case of fast sync.
    *
    */
    SyncItem* getFirstNewItem();

    /*
    * Return the next SyncItem of new. It is used in case of fast sync.
    */
    SyncItem* getNextNewItem();

    /*
    * Return the first SyncItem of updated. It is used in case of fast sync.
    */
    SyncItem* getFirstUpdatedItem();

    /*
    * Return the next SyncItem of updated. It is used in case of fast sync.
    */
    SyncItem* getNextUpdatedItem();

    /*
    * Return the first SyncItem of deleted. It is used in case of fast sync.
    */
    SyncItem* getFirstDeletedItem();

    /*
    * Return the next SyncItem of deleted. It is used in case of fast sync.
    */
    SyncItem* getNextDeletedItem();

    /*
    * Return the key of the first Item of All,
    * for refresh-from-server sync.
    */
    SyncItem* getFirstItemKey();

    /*
    * Return the key of the next Item of All,
    * for refresh-from-server sync.
    */
    SyncItem* getNextItemKey();


    /*
     * Add the item from the server to the client.
     * Sets the SyncItem key
     */
    int addItem(SyncItem& item);

    /*
     * Update the item from the server to the client.
     */
    int updateItem(SyncItem& item);

    /*
     * Delete the item from the server to the client.
     */
    int deleteItem(SyncItem& item);

};

#endif
